#!/usr/bin/python

import argparse
import sys, os 

descr = """This program shows the moving image of a circle of radius r centerd at 
the origin under the transformation given by the identity minus the time 
translation defined by the parameter-dependent ODE whose right-hand-side is given 
by the user in the form of a FORTRAN (90 or higher) routine (its name must be 
passed as first argument), and with parameter varying between given bounds. (More 
fun than useful.) The other parameters must be specified as optional arguments 
(see below).
"""

parser = argparse.ArgumentParser(description=descr)
parser.add_argument("f", metavar="RHS", action="store",
                    help="RightHandSide routine name (extension included)")
parser.add_argument("-r", "--radius", action="store", type=str, dest="r", default="1.0", 
                    help="Radius of the circle to be transformed (default: 1.0)")
parser.add_argument("-n", "--npunti", action="store", type=int, dest="n",default='100', 
                    help="# of points to be used (default = 10000)")
parser.add_argument("-T", "--Time", action="store", type=str, dest="T",default="1.0", 
                    help="Time translation (default: 1.0)")
parser.add_argument("-nf", "--nframes", action="store", type=int, dest="nframes",default="100", 
                    help="number of frames (default: 100)")
parser.add_argument("-lm", "--lambdamin", action="store", type=str, dest="lamin",default="1.0", 
                    help="minimum parameter in the function (default: 1.0)")
parser.add_argument("-lM", "--lambdamax", action="store", type=str, dest="lamax",default="1.0", 
                    help="Maximum parameter in the function (default: 1.0)")
parser.add_argument("-xr", "--xrange", action="store",nargs='+',type=str, dest="xrng",
                    help="x variable range")
parser.add_argument("-yr", "--yrange", action="store",nargs='+',type=str, dest="yrng",
                    help="y variable range")
parser.add_argument("--scale", action="store_true", dest="scala", 
                  help="Logaritmic radial scaling of plot (not active by default)")                 
arguments = parser.parse_args()
equaz = './equations/'+arguments.f
toBeCompiled = "f95 ./src/disegna2d.f95 ./src/Ode2.f90 %s -O3 -fopenmp -o ./bin/drawit -J ./bin" % equaz
failure = os.system(toBeCompiled)
if failure:
  print 'Compilation Error'
  quit()
else:
  if not os.path.exists('./tmp/'):
    os.system('mkdir ./tmp')
  os.system('rm ./tmp/* 2> /dev/null')
  if arguments.scala: 
    scala = ".TRUE."
  else:
    scala = ".FALSE."
  f = open('./tmp/disegno.gnuplot', 'w')
  f.write("set terminal gif animate delay 5\n")
  if arguments.xrng is not None: 
    xrang = arguments.xrng
    f.write("set xrange [%g:%g]\n" % (eval(xrang[0]) , eval(xrang[1])))
  if arguments.yrng is not None: 
    yrang = arguments.yrng
    f.write("set yrange [%g:%g]\n" % (eval(yrang[0]) , eval(yrang[1])))
  f.write("set output 'movie.gif'\n")
  minl = eval(arguments.lamin)
  maxl = eval(arguments.lamax)
  for i in xrange(arguments.nframes):
    lam = minl+ i*(maxl-minl)/float(arguments.nframes)
    command = './bin/drawit %s %s %s %s %s %s' % (eval(arguments.r), 
               arguments.n, eval(arguments.T), lam, scala, i)
    droupt = os.popen(command)       #
    risposta = droupt.readlines()    # Extracts degree
    err = droupt.close()             # from "drawit" output
    dati = risposta[0].split(',')    # 
    f.write("set title 'param = %2.5f,  %s'\n" % (lam,dati[0]))
    f.write("plot './tmp/disegno%s.dat' notitle with lines\n" % str(i).zfill(3))
  f.write("set term gif")
  f.close()
  os.system("gnuplot -persist ./tmp/disegno.gnuplot")
    
#EXAMPLE:
# python img_circ-movie.py test2a.f90 -lM 1.15 -lm 0.5 -r 0.5 -T 6.283185 -nf 400 -xr -1.7 2.0 -yr -2.0 2.0 -n 300

