#!/usr/bin/python

import argparse
import sys, os 

descr = """This program shows the static image of a circle of radius r centerd at 
the origin under the transformation given by the identity minus the time 
translation defined by the parameter-dependent ODE whose right-hand-side is given 
by the user in the form of a FORTRAN (90 or higher) routine (its name must be 
passed as first argument), and with parameter given (as argument). The other 
parameters must be specified as optional arguments (see below).
"""

parser = argparse.ArgumentParser(description=descr)
parser.add_argument("f", metavar="RHS", action="store",
                    help="RightHandSide routine name (extension included)")
parser.add_argument("-r", "--radius", action="store", type=str, dest="r", default="1.0", 
                    help="Radius of the circle to be transformed (default: 1.0)")
parser.add_argument("-n", "--npunti", action="store", type=int, dest="n",default='10000', 
                    help="# of points to be used (default = 10000)")
parser.add_argument("-T", "--Time", action="store", type=str, dest="T",default="1.0", 
                    help="Time translation (default: 1.0)")
parser.add_argument("-l", "--lambda", action="store", type=str, dest="lam",default="1.0", 
                    help="Parameter in the function (default: 1.0)")
parser.add_argument("--lines", action="store_false", dest="linee", 
                  help="Line-joined points in plot (default)")  
parser.add_argument("--dots", action="store_false", dest="linee", 
                  help="Lone points in plot")  
parser.add_argument("--scale", action="store_true", dest="scala", 
                  help="Logaritmic radial scaling of plot (not active by default)")                 
arguments = parser.parse_args()
equaz = './equations/'+arguments.f
toBeCompiled = "f95 ./src/disegna2d.f95 ./src/Ode2.f90 %s -O3 -fopenmp -o ./bin/drawit -J ./bin" % equaz
failure = os.system(toBeCompiled)
if failure:
  print 'Compilation Error'
  quit()
else:
  if not os.path.exists('./tmp/'):
    os.system('mkdir ./tmp')
  os.system('rm ./tmp/* 2> /dev/null')
  if arguments.scala: 
    scala = ".TRUE."
  else:
    scala = ".FALSE."
  err = os.system('./bin/drawit %s %s %s %s %s 0' % (eval(arguments.r), 
                  arguments.n, eval(arguments.T), eval(arguments.lam), scala))
  if err:
    print 'Execution error'
    quit()
  else:
    f = open('./tmp/disegno.gnuplot', 'w')
    if arguments.linee:
      f.write("plot './tmp/disegno000.dat' notitle with lines\n")
    else:  
      f.write("plot './tmp/disegno000.dat' notitle with dots\n")
    f.close()
    os.system("gnuplot -persist ./tmp/disegno.gnuplot")
    
#EXAMPLE:
#img_circ.py test1.f90 -r 0.8 -n 25000 -T 1.0 -l 1.1 --lines

