#!/usr/bin/python

import argparse
import sys, os 
from math import *

descr = """ This program shows the fixed points of the map given by the time 
translation defined by the parameter-dependent ODE whose right-hand-side is given 
by the user in the form of a FORTRAN (90 or higher) routine (its name must be 
passed as first argument), and with parameter varying within given bounds. The 
other parameters must be specified as optional arguments (see below).
"""

parser = argparse.ArgumentParser(description=descr)
parser.add_argument("f", metavar="RHS", action="store",
                    help="RightHandSide routine name (extension included)")
parser.add_argument("-x", "--xrange", action="store", type=str, dest="xrnge", 
                    help="range for x variable", nargs='+')
parser.add_argument("-y", "--yrange", action="store", type=str, dest="yrnge", 
                    help="range for y variable", nargs='+')
parser.add_argument("-l", "--lrange", action="store", type=str, dest="lrnge", 
                    help="range for l parameter", nargs='+')
parser.add_argument("-nx", "--nstepx", action="store", type=int, dest="nx",default='100', 
                    help="# of grid points on x (default = 100)")
parser.add_argument("-ny", "--nstepy", action="store", type=int, dest="ny",default='100', 
                    help="# of grid points on y (default = 100)")
parser.add_argument("-nl", "--nstepl", action="store", type=int, dest="nl",default='100', 
                    help="# of grid points on l (default = 100)")
parser.add_argument("-T", "--Time", action="store", type=str, dest="T",default="1.0", 
                    help="Time translation (default: 1.0)")
parser.add_argument("--dots", action="store_true", dest="linee", 
                  help="use points instead of standard symbols")                   
arg = parser.parse_args()
equaz = './equations/'+arg.f
toBeCompiled = "f95 ./src/disegna3dpar.f95 ./src/Ode4.f90 ./src/jacobiano3.f95 %s -O3 -fopenmp -o ./bin/drawit -J ./bin" % equaz
failure = os.system(toBeCompiled)
if failure:
  print 'Compilation Error'
  quit()
else:
  if not os.path.exists('./tmp/'):
    os.system('mkdir ./tmp')
  os.system('rm ./tmp/* 2> /dev/null')
  err = os.system('./bin/drawit %s %s %s %s %s %s %s %s %s %s' % (eval(arg.xrnge[0]), 
                  eval(arg.xrnge[1]),eval(arg.yrnge[0]),eval(arg.yrnge[1]),
                  eval(arg.lrnge[0]),eval(arg.lrnge[1]),arg.nx,arg.ny,arg.nl,
                  eval(arg.T)))
  if err:
    print 'Execution error'
    quit()
  else:
    f = open('./tmp/disegno.gnuplot', 'w')
    if arg.linee:
      f.write("splot './tmp/disegno.dat' notitle with dots\n")
    else:  
      f.write("splot './tmp/disegno.dat' notitle\n")
    f.close()
    os.system("gnuplot -persist ./tmp/disegno.gnuplot")
    
#EXAMPLE:
#python init_pts-3d.py test3.f90 -x -0.5 0.5 -y -0.5 0.5 -l 0.0 1.0 -nx 100 -ny 100 -nl 100 -T 2*pi


