 module rk4  !! Runge-Kutta method 4th order, 2D problems with one parameter
   implicit none
   integer,parameter,public :: ikind = selected_real_kind(p=16,r=20)
   contains

     pure function nexpt(deriv,tau,passo,lambda,xv)  result(y) !!Computes next step
       implicit none
       interface
         pure function deriv(t,lambda,xi)
           real,intent(in) :: t,lambda
           real,intent(in),dimension(2) :: xi
           real,dimension(2) :: deriv
           end function deriv
         end interface
       real,intent(in) :: tau,passo,lambda
       real,intent(in),dimension(2) :: xv
       real,dimension(2) :: y
       real(kind=ikind) :: th2
       real(kind=ikind),dimension(2) :: k1, k2, k3, k4
       real(kind=ikind),parameter :: onesixth = 1/6.
     
       th2 = tau + passo*0.5
       k1 = passo * deriv(tau,lambda,xv)
       k2 = passo * deriv(real(th2),lambda,real(xv+k1*0.5))
       k3 = passo * deriv(real(th2),lambda,real(xv+k2*0.5))
       k4 = passo * deriv(tau+passo,lambda,real(xv+k3))
       y = real(xv +(k1 + 2.*(k2 + k3) +k4)*onesixth)

       end function nexpt

     pure function oderk4(deriv,x0,periodo,lambda) result(y) !!Applies steps
       implicit none
       interface
         pure function deriv(t,lambda,xi)
           real,intent(in) :: t,lambda
           real,intent(in),dimension(2) :: xi
           real,dimension(2) :: deriv
           end function deriv
         end interface
       real,dimension(2) :: y
       real,intent(IN),dimension(2) :: x0
       real,intent(IN) :: lambda, periodo
       integer,parameter :: passi = 200
       integer :: i
       real :: t, step

       step = periodo/passi
       y=x0
       do i=0, passi-1
         t=i* step
         y = nexpt(deriv,t, step, lambda, y)
         end do
       end function oderk4
   
   end module rk4

 pure function f(xi,tt,lambda) !! Translation operator at time=tt minus identity
   use rk4
    implicit none
       interface
         pure function deriv(t,lambda,xi)  !! Function containing RHS of ODE
           real,intent(in) :: t,lambda
           real,intent(in),dimension(2) :: xi
           real,dimension(2) :: deriv
           end function deriv
         end interface
    real, dimension(2) :: f
    real, intent(IN) :: tt, lambda
    real, dimension(1:2),intent(IN) :: xi

    f= oderk4(deriv,xi,tt,lambda) - xi

end function f


   