 module rungekutta  !! Runge-Kutta methods, 2D problems with one parameter
   implicit none
   integer,parameter,public :: db = selected_real_kind(p=16,r=200)
   real(kind=db),parameter,public    :: atoll = 1.E-4_db, rtoll = 1.E-3_db, hmax = 0.25_db,&
                                        hmin = 0.00125_db,&
                                        hinit = 10._db*hmin, mins = 0.2_db, maxs = 10._db
   real(kind=db),parameter,private :: c1=4._db/15._db, c2=4._db/75._db, c3=16._db/75._db, &
                             d1=2._db/3._db, d2=5._db/6._db, d3=8._db/3._db, d4= 5._db/2._db, &
                             e1=165._db/64._db, e2=55._db/6._db, e3= 425._db/64, e4=85._db/96._db, &
                             f1=12._db/5._db, f2=4015._db/612._db, f3=11._db/36._db,&  
                             f4=88._db/255._db, &
                             g1=8263._db/15000._db, g2=124._db/75._db, g3=643._db/680._db, &
                             g4=81._db/250._db, g5=2484._db/10625._db, &
                             h1=3501._db/1720._db, h2=300._db/43._db, h3=297275._db/52632._db, &
                             h4=319._db/2322._db, h5=24068._db/84065._db, h6=3850._db/26703._db,&
                             w1=125._db/17952._db, w2=12._db/1955._db, w3=3._db/44._db, &
                             w4=125._db/11592._db, w5=43._db/616._db, &
                             z1=3._db/40._db, z2=875._db/2244._db, z3=23._db/72._db, &
                             z4=264._db/1955._db, &
                             q1=1._db/6._db, q2=1._db/15._db, q3=1._db/160._db, q4=1._db/144._db
    integer,parameter,public :: passi = 30, passi4 = 150
   contains

   pure function dRKV56_adaptive(deriv,x0,periodo,lambda) result(y) 
       implicit none
       interface
         pure function deriv(t,lambda,xi)
           real,intent(in) :: t,lambda
           real,intent(in),dimension(2) :: xi
           real,dimension(2) :: deriv
           end function deriv
         end interface
       real,dimension(2) :: y
       real,intent(IN),dimension(2) :: x0
       real,intent(IN) :: lambda, periodo
       real(kind=db),dimension(2) :: k1, k2, k3, k4, k5, k6, k7, k8, yy, yy_new
       real(kind=db) :: R, q, t0, h, tzph, errass

       h = hinit
       t0 = 0._db
       yy = x0
       mcycle: do !while ( t0 .lt. periodo )
        attempts: do
         tzph = t0+h
         k1 = deriv(real(t0),lambda,real(yy))
         k2 = deriv(real(t0+h*q1),lambda,real(yy+k1*q1))
         k3 = deriv(real(t0+c1*h),lambda,real(yy+c2*k1+c3*k2))
         k4 = deriv(real(t0+d1*h),lambda,real(yy+d2*k1-d3*k2+d4*k3))
         k5 = deriv(real(t0+d2*h),lambda,real(yy-e1*k1+e2*k2-e3*k3+e4*k4))
         k6 = deriv(real(tzph),lambda,real(yy+f1*k1-8._db*k2+f2*k3-f3*k4+f4*k5))
         k7 = deriv(real(t0+h*q2),lambda,real(yy-g1*k1+g2*k2-g3*k3-g4*k4+g5*k5))
         k8 = deriv(real(tzph),lambda, real(yy+h1*k1-h2*k2+h3*k3-h4*k4+h5*k5+h6*k7))
         yy_new = yy+h*(z1*k1+z2*k3+z3*k4+z4*k5+w4*k7+w5*k8)
         errass = maxval(abs(k1*q3+w1*k3-k4*q4+w2*k5+w3*k6-w4*k7-w5*k8))*h
         R = max(atoll,errass/(atoll+rtoll*max(maxval(abs(yy_new)),maxval(abs(yy)))))
         q =max(mins,min(maxs,0.85_db * R**(-0.2_db)))
         if ((R .lt. 1._db).or.(h .le. hmin)) then
           t0 = tzph
           yy = yy_new
           h=min(q*h,hmax,periodo-t0)
           exit attempts
         else 
           h=min(max(q,1._db)*h,hmin)
           end if
         end do attempts
         if (t0 .ge. periodo) exit mcycle
         end do mcycle
         y = yy
       end function dRKV56_adaptive



   pure function d_dRKV56_adp(rhs,x0,periodo,lambda) result(yy) 
       implicit none
       interface
         pure function rhs(t,lambda,xi)
           integer,parameter :: dbl = selected_real_kind(p=16,r=20)
           real(kind=dbl),intent(in) :: t,lambda
           real(kind=dbl),intent(in),dimension(2) :: xi
           real(kind=dbl),dimension(2) :: rhs
           end function rhs
         end interface
       real(kind=db),dimension(2) :: yy
       real(kind=db),intent(IN),dimension(2) :: x0
       real(kind=db),intent(IN) :: lambda, periodo
       real(kind=db),dimension(2) :: k1, k2, k3, k4, k5, k6, k7, k8
       real(kind=db) :: R, q, t0, h, tzph
       integer :: cn
       cn=1
       h = hmax
       t0 = 0._db
       yy = x0
       mcycle: do !while ( t0 .lt. periodo )
         tzph = t0+h
         k1 = h * rhs(t0,lambda,yy)
         k2 = h * rhs(t0+h*q1,lambda,yy+k1*q1)
         k3 = h * rhs(t0+c1*h,lambda,yy+c2*k1+c3*k2)
         k4 = h * rhs(t0+d1*h,lambda,yy+d2*k1-d3*k2+d4*k3)
         k5 = h * rhs(t0+d2*h,lambda,yy-e1*k1+e2*k2-e3*k3+e4*k4)
         k6 = h * rhs(tzph,lambda,yy+f1*k1-8._db*k2+f2*k3-f3*k4+f4*k5)
         k7 = h * rhs(t0+h*q2,lambda,yy-g1*k1+g2*k2-g3*k3-g4*k4+g5*k5)
         k8 = h * rhs(tzph,lambda, yy+h1*k1-h2*k2+h3*k3-h4*k4+h5*k5+h6*k7)
         R = maxval(abs(k1*q3+w1*k3-k4*q4+w2*k5+w3*k6-w4*k7-w5*k8))/h
         q =0.87_db * (atoll / R)**(0.2_db)
         if (R .lt. atoll) then
           yy = yy+z1*k1+z2*k3+z3*k4+z4*k5+w4*k7+w5*k8
           t0 = tzph
           end if
           h=min(max(q,0.1_db),4._db)*h
           if (h .gt. hmax) then
              h = hmax
           elseif (h .lt. hmin) then
              h = hmin
              yy = yy+z1*k1+z2*k3+z3*k4+z4*k5+w4*k7+w5*k8
              t0 = tzph
             end if
         if ( t0+h .ge. periodo) then 
           h = periodo - t0
           k1 = h * rhs(t0,lambda,yy)
           k2 = h * rhs(t0+h*q1,lambda,yy+k1*q1)
           k3 = h * rhs(t0+c1*h,lambda,yy+c2*k1+c3*k2)
           k4 = h * rhs(t0+d1*h,lambda,yy+d2*k1-d3*k2+d4*k3)
           k5 = h * rhs(t0+d2*h,lambda,yy-e1*k1+e2*k2-e3*k3+e4*k4)
           k6 = h * rhs(t0+h,lambda,yy+f1*k1-8._db*k2+f2*k3-f3*k4+f4*k5)
           k7 = h * rhs(t0+h*q2,lambda,yy-g1*k1+g2*k2-g3*k3-g4*k4+g5*k5)
           k8 = h * rhs(t0+h,lambda,yy+h1*k1-h2*k2+h3*k3-h4*k4+h5*k5+h6*k7)
           yy = yy+z1*k1+z2*k3+z3*k4+z4*k5+w4*k7+w5*k8
           exit mcycle
           endif
           cn=cn+1
         end do mcycle
         !print*, cn
       end function d_dRKV56_adp




   pure function drkv56_fixStep(deriv,x0,periodo,lambda) result(y) 
       implicit none
       interface
         pure function deriv(t,lambda,xi)
           real,intent(in) :: t,lambda
           real,intent(in),dimension(2) :: xi
           real,dimension(2) :: deriv
           end function deriv
         end interface
       real,dimension(2) :: y
       real,intent(IN),dimension(2) :: x0
       real,intent(IN) :: lambda, periodo
       real(kind=db),dimension(2) :: k1, k2, k3, k4, k5, k6, k7, k8, yy
       real(kind=db) :: h, tzph, t0 
       integer :: i
       t0=0._db
       yy=x0
       h=periodo/passi
       do i=0,passi-1 
         tzph = t0+h
         k1 = h * deriv(real(t0),lambda,real(yy))
         k2 = h * deriv(real(t0+h*q1),lambda,real(yy+k1*q1))
         k3 = h * deriv(real(t0+c1*h),lambda,real(yy+c2*k1+c3*k2))
         k4 = h * deriv(real(t0+d1*h),lambda,real(yy+d2*k1-d3*k2+d4*k3))
         k5 = h * deriv(real(t0+d2*h),lambda,real(yy-e1*k1+e2*k2-e3*k3+e4*k4))
         !k6 = h * deriv(real(tzph),lambda,real(yy+f1*k1-8.*k2+f2*k3-f3*k4+f4*k5))
         k7 = h * deriv(real(t0+h*q2),lambda,real(yy-g1*k1+g2*k2-g3*k3-g4*k4+g5*k5))
         k8 = h * deriv(real(tzph),lambda,real(yy+h1*k1-h2*k2+h3*k3-h4*k4+h5*k5+h6*k7))      
         yy = yy+z1*k1+z2*k3+z3*k4+z4*k5+w4*k7+w5*k8
         t0=tzph
         enddo
         y=yy
       end function drkv56_fixStep



    pure function srkv56(deriv,x0,periodo,lambda) result(y) !!Applies steps
       implicit none
       interface
         pure function deriv(t,lambda,xi)
           real,intent(in) :: t,lambda
           real,intent(in),dimension(2) :: xi
           real,dimension(2) :: deriv
           end function deriv
         end interface
       real,dimension(2) :: y
       real,intent(IN),dimension(2) :: x0
       real,intent(IN) :: lambda, periodo
       integer,parameter :: passi = 20
       integer :: i
       real :: t, step

       step = periodo/passi
       y=x0
       do i=0, passi-1
         t=i* step
         y = nexpt56(deriv,t, step, lambda, y)
         end do
       contains
         pure function nexpt56(deriv,t0,h,lambda,x0)  result(y) !!Computes next step
           implicit none
        interface
          pure function deriv(t,lambda,xi)
            real,intent(in) :: t,lambda
            real,intent(in),dimension(2) :: xi
            real,dimension(2) :: deriv
            end function deriv
          end interface
           real,intent(in) :: t0,h,lambda
           real,intent(in),dimension(2) :: x0
           real,dimension(2) :: y
           real,dimension(2) :: k1, k2, k3, k4, k5, k6, k7, k8
           real R, q
           k1 = h * deriv(t0,lambda,x0)
           k2 = h * deriv(t0+h/6.,lambda,x0+k1/6.)
           k3 = h * deriv(t0+4.*h/15.,lambda,x0+4.*k1/75.+16.*k2/75.)
           k4 = h * deriv(t0+2.*h/3.,lambda,x0+5.*k1/6.-8.*k2/3.+5.*k3/2.)
           k5 = h * deriv(t0+5*h/6.,lambda,x0-165.*k1/64.+55.*k2/6.-425.*k3/64. &
                           +85.*k4/96.)
           k6 = h * deriv(t0+h,lambda,x0+12.*k1/5.-8.*k2+4015.*k3/612.-11.*k4/36. &
                           +88.*k5/255.)
           k7 = h * deriv(t0+h/15.,lambda,x0-8263.*k1/15000.+124.*k2/75.-643.*k3/680. &
                           -81.*k4/250.+2484.*k5/10625.)
           k8 = h * deriv(t0+h,lambda, x0+3501.*k1/1720.-300.*k2/43.+297275.*k3/52632. &
                           -319.*k4/2322.+24068.*k5/84065.+3850.*k7/26703.)
           y = x0+3.*k1/40.+875.*k3/2244.+23.*k4/72.+264.*k5/1955. &
                           +125.*k7/11592.+43.*k8/616.
           end function nexpt56
       end function srkv56
   

     pure function nexpt(deriv,tau,passo,lambda,xv)  result(y) !!Computes next step
       implicit none
       interface
         pure function deriv(t,lambda,xi)
           real,intent(in) :: t,lambda
           real,intent(in),dimension(2) :: xi
           real,dimension(2) :: deriv
           end function deriv
         end interface
       real,intent(in) :: tau,passo,lambda
       real,intent(in),dimension(2) :: xv
       real,dimension(2) :: y
       real(kind=db) :: th2
       real(kind=db),dimension(2) :: k1, k2, k3, k4
       real(kind=db),parameter :: onesixth = 1/6.
     
       th2 = tau + passo*0.5
       k1 = passo * deriv(tau,lambda,xv)
       k2 = passo * deriv(real(th2),lambda,real(xv+k1*0.5))
       k3 = passo * deriv(real(th2),lambda,real(xv+k2*0.5))
       k4 = passo * deriv(tau+passo,lambda,real(xv+k3))
       y = real(xv +(k1 + 2.*(k2 + k3) +k4)*onesixth)

       end function nexpt

     pure function oderk4(deriv,x0,periodo,lambda) result(y) !!Applies steps
       implicit none
       interface
         pure function deriv(t,lambda,xi)
           real,intent(in) :: t,lambda
           real,intent(in),dimension(2) :: xi
           real,dimension(2) :: deriv
           end function deriv
         end interface
       real,dimension(2) :: y
       real,intent(IN),dimension(2) :: x0
       real,intent(IN) :: lambda, periodo
       integer :: i
       real :: t, step

       step = periodo/passi4
       y=x0
       do i=0, passi4-1
         t=i* step
         y = nexpt(deriv,t, step, lambda, y)
         end do
       end function oderk4
   

   end module rungekutta


   