!! Draws the image of a circle of radius r centered at the origin under the flow
!! defined by the ODE in the 'deriv' function and gives a rough estimate of the 
!! fixed point index of the flow.
program disegno
  implicit none
  interface
    function f(xi,t,l)
      real, dimension(2) :: f
      real, intent(IN) :: t,l
      real, dimension(2), intent(IN) :: xi
      end function f
    end interface
  integer,parameter :: ikind = selected_real_kind(p=16,r=20)
  real, parameter :: twopi=8.*atan(1.)
  integer :: i, n, deg, nome
  real :: r, normsq, thetastep, angle, period, lambda
  real(kind=ikind) :: accum=0.0
  real(kind=ikind),dimension(2) ::fvdif
  real(kind=ikind), allocatable, dimension(:,:) :: fvals, fplot
  logical :: scala, toosmall = .false.
  logical,allocatable,dimension(:) :: takeit 
  character (len=15) :: arg0
  character (len=20) :: fname

  call getarg(1,arg0)
  read(arg0,*) r  
  call getarg(2,arg0)
  read(arg0,*) n
  call getarg(3,arg0)
  read(arg0,*) period
  call getarg(4,arg0)
  read(arg0,*) lambda
  call getarg(5,arg0)
  read(arg0,*) scala
  call getarg(6,arg0)
  read(arg0,*) nome

  write(fname,"(A13,I0.3,A4)") "./tmp/disegno", nome,".dat"

  thetastep = twopi/float(n)
  allocate(takeit(0:n))
  allocate(fplot(0:n,2))
  allocate(fvals(-1:n+1,2))
  open(unit=3,file=fname,form='formatted',status='new',asynchronous='yes')
!$OMP parallel PRIVATE(i,angle,normsq,fvdif) &
!$OMP SHARED(r,thetastep,n,fvals,period,lambda,scala,toosmall) reduction(+:accum)
!$omp workshare
  takeit = .true.
!$omp end workshare nowait
!$OMP do 
  do i=0,n
    angle = i*thetastep
    fvals(i,:)=f((/ r*cos(angle) , r*sin(angle) /), period, lambda)
    end do
!$OMP end do NOWAIT
!$OMP single
  fvals(-1,:)=fvals(n,:)
  fvals(n+1,:)=fvals(0,:)
!$omp end single
!$omp barrier
!$OMP do
  main_cycle: do i=0,n
    normsq = dot_product(fvals(i,:),fvals(i,:))
    if ((normsq .gt. 1.E15) .or. (isnan(normsq))) then
      takeit(i) = .false.
      cycle main_cycle
      endif
    if (normsq .lt. 1.E-10) then
!$omp critical
      toosmall = .true.
!$omp end critical
      normsq = normsq+1.e-10
      endif
    fvdif=fvals(i-1,:)-fvals(i+1,:)
    accum = accum + (fvdif(1)*fvals(i,2)-fvdif(2)*fvals(i,1))/normsq
    if (scala) then
      normsq = sqrt(normsq)
      fplot(i,:) = fvals(i,:)*log10(1.+normsq)/normsq
    else
      fplot(i,:) = fvals(i,:)
      endif
    end do main_cycle
!$OMP end do
!$omp barrier
!$OMP do ordered
    do i=0,n
      if (takeit(i)) then 
!$omp ordered
        write(3,*) fplot(i,:)
!$omp end ordered
        endif
      end do
!$omp end do
!$OMP end parallel
  accum = accum/(2.*twopi)
  deg = nint(accum)
  print '(A,I3,A,F10.5)', "Degree: ", deg, ", Error: ",abs(deg-accum)
  if ((toosmall) .or. (.not. all(takeit))) then 
    print*, "Degree not to be trusted!"
    endif
   
  close(3)

  deallocate(fvals)
  deallocate(fplot)
  end program disegno
    