!! Analyzes a 3D-grid, looking for initial points of periodic solutions. Search
!! is refined with Newton iteration when promising points in the grid are found.
program disegno
  use Jacobian
  implicit none
  interface
    pure function fbis(xi,t,l)
      real, dimension(2) :: fbis
      real, intent(IN) :: t,l
      real, dimension(2), intent(IN) :: xi
      end function fbis
    end interface
  real, parameter :: overlap = 0.65 !! 30% overlap of nearby cells: 0.5= no overlap
  real, parameter :: toll = 2.e-3  !! what we really consider a zero
  integer, parameter :: maxcnt = 10 !! maximum number of Newton iterations
  integer :: i,j,k,cnt,punti, nstepx,nstepy,nstepl
  real,dimension(2) :: x, x0, y
  real,dimension(2,2) :: inv
  real :: l,stepx,stepy,stepl,maxstx,maxsty, tempo,a0,a1,b0,b1,l0,l1
  character (len=15) :: arg0

  call getarg(1,arg0)
  read(arg0,*) a0  !Left bound for x  
  call getarg(2,arg0)
  read(arg0,*) a1  !Right bound for x
  call getarg(3,arg0)
  read(arg0,*) b0  !Left bound for y  
  call getarg(4,arg0)
  read(arg0,*) b1  !Right bound for y
  call getarg(5,arg0)
  read(arg0,*) l0  !Smallest bound for parameter l
  call getarg(6,arg0)
  read(arg0,*) l1  !Largest bound for parameter l
  call getarg(7,arg0)
  read(arg0,*) nstepx  !number of x-intervals
  call getarg(8,arg0)
  read(arg0,*) nstepy  !number of y-intervals
  call getarg(9,arg0)
  read(arg0,*) nstepl  !number of l-intervals
  call getarg(10,arg0)
  read(arg0,*) tempo  !time for time translation

  stepx = (a1-a0)/nstepx
  stepy = (b1-b0)/nstepy
  stepl = (l1-l0)/nstepl
  
  maxstx = overlap*stepx
  maxsty = overlap*stepy
  punti = 0

  open(unit=3,file='./tmp/disegno.dat',form='formatted',status='new',asynchronous='yes')
!$OMP parallel do PRIVATE(i,j,k,x,y,l,cnt,x0,inv) SHARED(stepx,stepy,stepl) SCHEDULE(dynamic)
  do i=0,nstepl
    l=i*stepl+l0
    do j=0,nstepx
      x0(1)=stepx*j+a0
      do k=0,nstepy
        x0(2)=stepy*k+b0
        y = fbis(x0,tempo,l)
        if ( (abs(y(1)) .lt. stepx).and. (abs(y(2)) .lt. stepy)) then
          x=x0
          newtonit: do cnt=1,maxcnt ! Newton iteration: search for a zero in each cell
            if (all(abs(y) .lt. toll)) then
!$OMP CRITICAL     
              punti = punti + 1
              write(3,*) x(1),x(2),l 
!$OMP END CRITICAL
              exit newtonit !! zero found and saved, cycle aborted
            else
              inv = invJac(fbis,x,tempo,l)
              if (all(abs(inv) .lt. 1.e-6)) exit newtonit
              x = x - matmul(inv, y) 
              !! aborts search if x is too far out of cell
              if ((abs(x(1)-x0(1)) .ge. maxstx) .or. (abs(x(2)-x0(2)) .ge. maxsty)) exit newtonit
              y = fbis(x,tempo,l)
            end if 
          end do newtonit
        endif
      end do
    end do
  end do
!$OMP end parallel do

  close(3)
  print*, punti,' points found.'

  end program disegno


pure function fbis(xi,tt,lambda) !! Translation operator at time=tt minus identity
   use rungekutta
   implicit none
   interface
     pure function deriv(t,lambda,xi)  !! Function containing RHS of ODE
       real,intent(in) :: t,lambda
       real,intent(in),dimension(2) :: xi
       real,dimension(2) :: deriv
       end function deriv
     end interface
   real, dimension(2) :: fbis
   real, intent(IN) :: tt, lambda
   real, dimension(2),intent(IN) :: xi

   fbis= drkv56_adaptive(deriv,xi,tt,lambda) - xi
end function fbis

    