module Jacobian !! Two-dimensional stuff
  integer,parameter,private :: ikind = selected_real_kind(16)
  real(kind=ikind), parameter,private :: eps = 1.E-4, deps=2.E-4
  contains
  
pure function invJac(funz, x, t,l) result(J) !!Computes the inverse of the Jacobian
  implicit none
  interface
    pure function funz(x,t,l)
      real,intent(IN),dimension(2) :: x 
      real,intent(IN) :: t,l
      real,dimension(2) :: funz
    end function funz
  end interface
  real,dimension(2),intent(IN) :: x
  real,intent(IN) :: t,l
  real,dimension(2,2) :: xp,xm,J
  real(kind=ikind), dimension(2,2) :: M
  real(kind=ikind) :: det
  integer :: i
  xp = reshape( spread(x,2,2),shape(xp))
  xm = xp
  forall (i=1:2)
    xp(i,i)=xp(i,i)+eps
    xm(i,i)=xm(i,i)-eps
    end forall
  forall ( i=1:2 )  
    M(:,i)=(funz(xp(:,i),t,l)-funz(xm(:,i),t,l)) /deps
    end forall !! M contains the Jacobian
  det = (M(1,1)*M(2,2)-M(2,1)*M(1,2))
  if (det .lt. 1e-7) then
     J = 0. !! Sets inverse to 0 when M is not invertible. Incorrect but 
            !! effective, given the usage of this function: one needs not mess
            !! with error messages.
  else
    J = reshape( (/ M(2,2) , -M(2,1), -M(1,2), M(1,1) /), shape(J))/det 
  endif
end function invJac

end module Jacobian


