module Jacobian !! Two-dimensional stuff
  integer,parameter,private :: dbl = selected_real_kind(16)
  real,parameter,private :: eps = 1.E-3, deps = 0.5E3
  real,parameter,private,dimension(2,2) :: epa = reshape((/eps, 0., &
                                                            0., eps/), (/2,2/))
  contains
  
function invJac(funz, x, t,l) result(J) !!Computes the inverse of the Jacobian
  implicit none
  interface
    pure function funz(x,t,l)
      real,intent(IN),dimension(2) :: x 
      real,intent(IN) :: t,l
      real,dimension(2) :: funz
    end function funz
  end interface
  real,dimension(2),intent(IN) :: x
  real,intent(IN) :: t,l
  real,dimension(2,2) :: J 
  real :: det

  J(:,1)= funz(x+epa(:,1),t,l)-funz(x-epa(:,1),t,l)
  J(:,2)=funz(x+epa(:,2),t,l)-funz(x-epa(:,2),t,l)
  det = (J(1,1)*J(2,2)-J(2,1)*J(1,2))*deps
  if (abs(det) .lt. 1.e-7) then
     J = 0. !! Sets inverse to 0 when M is not invertible. Incorrect but 
            !! effective, given the usage of this function: one needs not mess
            !! with error messages.
  else
    J = reshape( (/ J(2,2) , -J(2,1), -J(1,2), J(1,1) /), shape(J))/det
  endif
end function invJac

end module Jacobian


